from datetime import datetime
from friday.types import FridayLogRecord, NamespaceAndTopics, Level
from friday.utils import datetime_to_string, parse_friday_log
from typing import List, Optional
from urllib.parse import urljoin
import json
import requests


class Aggregator:
    def __init__(self, friday_endpoint: str):
        self.friday_endpoint = friday_endpoint

    # TODO: infer these types from QueryInput
    def query(
        self,
        namespace_and_topics: Optional[List[NamespaceAndTopics]] = None,
        level: Optional[Level] = None,
        before: Optional[datetime] = None,
        after: Optional[datetime] = None,
        limit: Optional[int] = None,
    ) -> List[FridayLogRecord]:
        data = {}
        if namespace_and_topics:
            data["namespacesAndTopics"] = namespace_and_topics
        if level:
            data["level"] = level
        if before:
            data["before"] = datetime_to_string(before)
        if after:
            data["after"] = datetime_to_string(after)
        if limit:
            data["limit"] = limit

        resp = requests.get(
            urljoin(self.friday_endpoint, "getLogs"), params={"input": json.dumps(data)}
        )

        data = resp.json()["result"]["data"]
        parsed_data = [parse_friday_log(log) for log in data]
        return parsed_data



from typing import TypedDict, List, Optional, Literal
from datetime import datetime
from enum import Enum


class NamespaceAndTopics(TypedDict):
    namespace: str
    topic: str


Level = Literal["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"]


class QueryInput(TypedDict):
    namespace_and_topics: List[NamespaceAndTopics]
    level: Optional[Level]
    before: Optional[datetime]
    after: Optional[datetime]
    limit: Optional[int]


class FridayLogRecord(TypedDict):
    id: int
    namespace: str
    topic: str
    level: Level
    data: str
    timestamp: datetime


DATETIME_FORMAT = "%Y-%m-%dT%H:%M:%S.%fZ"



from datetime import datetime


from friday.types import DATETIME_FORMAT, FridayLogRecord, Level


def datetime_to_string(value: datetime):

    "Converts the datetime object to string format used by friday database"

    return value.strftime(DATETIME_FORMAT)


def datetime_from_string(value: str):

    "Converts the timestamp string to datetime object"

    return datetime.strptime(value, DATETIME_FORMAT)


def parse_friday_log(data: dict) -> FridayLogRecord:

    return FridayLogRecord(
        id=data["id"],
        namespace=data["namespace"],
        topic=data["topic"],
        level=data["level"],
        data=data["data"],
        timestamp=datetime_from_string(data["timestamp"]),
    )
